@@ -1,7 +1,7 @@
 /*
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998, 1999 Alfredo K. Kojima
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -1641,9 +1641,11 @@ wMouseMoveWindow(WWindow *wwin, XEvent *ev)
                                moveData.realX - wwin->frame_x,
 			       moveData.realY - wwin->frame_y);
 		}
+	    /*} else {
+		WMHandleEvent(&event); this causes problems needs fixing */
 	    }
 	    break;
-	    
+
 	 case MotionNotify:
 	    if (started) {
 		updateWindowPosition(wwin, &moveData,
